package  
{
	import org.flixel.*;
	
	/**
	 * ...
	 * @author Switchbreak
	 */
	public class Enemy extends FlxSprite
	{
		// Embed sound effects and images
		[Embed (source = "../sounds/fling.mp3")] private var FlingSnd:Class;
		[Embed (source = "../sounds/enemy-hit.mp3")] private var ExplosionSnd:Class;
		[Embed (source = "../images/Enemy.png")] private var EnemyImg:Class;
		
		// Constants
		private const ENEMY_RADIUS_SQUARED:Number		= 400;
		private const PRODUCER_RADIUS_SQUARED:Number	= 1521;
		private const PRODUCER_DAMAGE:Number			= 7;
		private const ENEMY_SIZE:uint					= 20;
		private const ENEMY_COLOR:uint					= 0xffff0000;
		private const ENEMY_SPEED:Number				= 100;
		private const ENEMY_SCORE:uint					= 100;
		private const THROW_SPEED:Number				= 1000;
		private const THROW_TIME:Number					= 5;
		private const DIRECTION_TIME:Number				= 0.2;
		private const TRAIL_PARTICLES:uint				= 5;
		private const TRAIL_RANGE:uint					= 20;
		private const EXPLODE_PARTICLES:uint			= 5;
		
		private const STATE_IDLE:uint	= 0;
		private const STATE_THROW:uint	= 1;
		
		// Enemy attributes
		private var directionTimer:Number;
		private var throwTimer:Number;
		private var state:uint = STATE_IDLE;
		private var trail:FlxEmitter;
		
		// Constructor
		public function Enemy( X:Number, Y:Number ) 
		{
			// Call Sprite constructor
			super( X, Y, EnemyImg );
			
			// Create exhaust trail
			trail = new FlxEmitter( x + frameWidth / 2, y + frameHeight / 2 );
			trail.gravity = 0;
			trail.delay = 0.05;
			trail.setRotation( 0, 0 );
			for ( var i:uint = 0; i < TRAIL_PARTICLES; i++ )
			{
				var newParticle:FlxSprite = new FlxSprite( x, y );
				newParticle.createGraphic( 1, 1, ENEMY_COLOR );
				trail.add( newParticle );
			}
			FlxG.state.add( trail );
			trail.start( false );
			
			// Get the enemy started moving around
			angle = Math.random() * 360;
			ChangeDirection();
		}
		
		// Update enemy state
		override public function update():void
		{
			// Choose action based on enemy state
			switch( state )
			{
				case STATE_IDLE:
					// Count down to next direction change
					trail.x = x + frameWidth / 2;
					trail.y = y + frameHeight / 2;
					directionTimer -= FlxG.elapsed;
					if ( directionTimer <= 0 )
						ChangeDirection();
					break;
				case STATE_THROW:
					// Check for collisions with all the other enemies
					for ( var i:uint = 0; i < PlayState.enemies.members.length; i++ )
					{
						var enemy:Enemy = PlayState.enemies.members[i];
						if ( enemy != null && enemy != this )
						{
							// Get the distance between the other enemy and this one
							var deltaX:Number = (enemy.x + enemy.frameWidth / 2) - (x + frameWidth / 2);
							var deltaY:Number = (enemy.y + enemy.frameHeight / 2) - (y + frameHeight / 2);
							
							if ( deltaX * deltaX + deltaY * deltaY <= ENEMY_RADIUS_SQUARED )
							{
								// If they have collided, explode both sprites
								Explode( false );
								enemy.Explode( true );
							}
						}
					}
					
					// Check for collisions with all the other enemies
					for ( i = 0; i < PlayState.producers.members.length; i++ )
					{
						var producer:Producer = PlayState.producers.members[i];
						if ( producer != null )
						{
							// Get the distance between the other enemy and this one
							deltaX = (producer.x + producer.frameWidth / 2) - (x + frameWidth / 2);
							deltaY = (producer.y + producer.frameHeight / 2) - (y + frameHeight / 2);
							
							if ( deltaX * deltaX + deltaY * deltaY <= PRODUCER_RADIUS_SQUARED )
							{
								// If they have collided, explode both sprites
								Explode( false );
								producer.hurt( PRODUCER_DAMAGE );
							}
						}
					}
					
					// Countdown timer until enemy resumes normal motion
					throwTimer -= FlxG.elapsed;
					if ( throwTimer <= 0 )
					{
						trail.start( false );
						state = STATE_IDLE;
						ChangeDirection();
					}
					
					break;
			}
			
			// Check for collision with the playfield boundaries
			if ( x < 0 )
			{
				x = 0;
				velocity.x = Math.abs( velocity.x );
				angle = FlxU.getAngle( velocity.x, velocity.y ) + 90;
			}
			if ( y < 0 )
			{
				y = 0;
				velocity.y = Math.abs( velocity.y );
				angle = FlxU.getAngle( velocity.x, velocity.y ) + 90;
			}
			if ( x > PlayState.PLAYFIELD_SIZE - ENEMY_SIZE )
			{
				x = PlayState.PLAYFIELD_SIZE - ENEMY_SIZE;
				velocity.x = -Math.abs( velocity.x );
				angle = FlxU.getAngle( velocity.x, velocity.y ) + 90;
			}
			if ( y > PlayState.PLAYFIELD_SIZE - ENEMY_SIZE )
			{
				y = PlayState.PLAYFIELD_SIZE - ENEMY_SIZE;
				velocity.y = -Math.abs( velocity.y );
				angle = FlxU.getAngle( velocity.x, velocity.y ) + 90;
			}
			
			super.update();
		}
		
		// Set the direction to a random angle and start moving that way
		private function ChangeDirection():void
		{
			angle += (Math.random() * 20) - 10
			
			FlxU.rotatePoint( 0, -ENEMY_SPEED, 0, 0, angle, velocity );
			
			trail.setXSpeed( -velocity.x + TRAIL_RANGE, -velocity.x - TRAIL_RANGE );
			trail.setYSpeed( -velocity.y + TRAIL_RANGE, -velocity.y - TRAIL_RANGE );
			
			directionTimer = DIRECTION_TIME;
		}
		
		// Explode function
		public function Explode( score:Boolean = true ):void
		{
			PlayState.enemies.remove( this );
			trail.kill();
			kill();
			
			// Create explosion particle effect
			var explode:FlxEmitter = new FlxEmitter( x + frameWidth / 2, y + frameHeight / 2 );
			explode.gravity = 0;
			explode.setRotation( 0, 0 );
			for ( var i:uint = 0; i < EXPLODE_PARTICLES; i++ )
			{
				var newParticle:FlxSprite = new FlxSprite( x, y );
				newParticle.createGraphic( 5, 5, ENEMY_COLOR );
				explode.add( newParticle );
			}
			FlxG.state.add( explode );
			explode.start();
			
			if ( score )
			{
				// Player gets points for hitting an enemy
				FlxG.play( ExplosionSnd );
				FlxG.score += ENEMY_SCORE;
				FlxG.state.add( new ScoreText( x, y, 100, "+" + ENEMY_SCORE.toString() ) );
			}
		}
		
		// Throw function
		public function Throw( throwAngle:Number ):void
		{
			trail.stop();
			FlxG.play( FlingSnd );
			angle = throwAngle;
			FlxU.rotatePoint( 0, -THROW_SPEED, 0, 0, angle, velocity );
			state = STATE_THROW;
			throwTimer = THROW_TIME;
		}
		
	}

}