package  
{
	import flash.geom.Point;
	import org.flixel.*;
	
	/**
	 * ...
	 * @author Switchbreak
	 */
	public class PlayState extends FlxState
	{
		// Embed images
		[Embed( source = '../images/HealthBar.png' )] private var HealthImg:Class;
		
		// Constants
		public static const PLAYFIELD_SIZE:uint	= 800;
		private const STARTING_ENEMIES:uint		= 15;
		private const STARTING_PRODUCERS:uint	= 2;
		private const PRODUCER_DISTANCE:uint	= 300;
		private const SAFE_ZONE_SQUARED:Number	= 10000;
		
		// PlayState attributes
		public static var enemies:FlxGroup;
		public static var producers:FlxGroup;
		
		public static var wave:uint = 1;
		
		private var playerShip:PlayerShip;
		private var HUD:FlxGroup;
		private var scoreDisplay:FlxText;
		private var healthDisplay:FlxSprite;
		private var gameOverText1:FlxText;
		private var gameOverText2:FlxText;
		private var fadeTimer:Number = 0;
		
		override public function create():void
		{
			// Create the starfield in the background
			add( new StarField() );
			
			// Create player ship in the center of the playfield
			playerShip = new PlayerShip( PLAYFIELD_SIZE / 2, PLAYFIELD_SIZE / 2 );
			add( playerShip );
			FlxG.follow( playerShip );
			FlxG.followBounds( 0, 0, PLAYFIELD_SIZE, PLAYFIELD_SIZE );
			playerShip.flicker();
			
			// Create enemy producers
			producers = new FlxGroup();
			add( producers );
			for ( var i:uint = 0; i < STARTING_PRODUCERS + wave; i++ )
			{
				var newPos:FlxPoint = FlxU.rotatePoint( playerShip.x, playerShip.y - PRODUCER_DISTANCE, playerShip.x, playerShip.y, (360 / (STARTING_PRODUCERS + wave)) * i );
				
				producers.add( new Producer( newPos.x, newPos.y ) );
			}
			
			// Create starting enemies
			enemies = new FlxGroup();
			add( enemies );
			for ( i = 0; i < STARTING_ENEMIES + (wave * 5); i++ )
				enemies.add( SpawnEnemy() );
			
			// Create HUD
			HUD = new FlxGroup();
			HUD.scrollFactor.x = HUD.scrollFactor.y = 0;
			add( HUD );
			
			// Create score display on HUD
			scoreDisplay = new FlxText( FlxG.width - 100, 0, 100 );
			scoreDisplay.alignment = "right";
			scoreDisplay.size = 20;
			HUD.add( scoreDisplay, true );
			
			// Create health bar display on HUD
			var healthText:FlxText = new FlxText( 3, 0, 100, "Health:" );
			healthText.size = 20;
			HUD.add( healthText, true );
			HUD.add( new FlxSprite( 5, 28, HealthImg ), true );
			healthDisplay = new FlxSprite( 6, 29 );
			healthDisplay.createGraphic( 1, 18, 0xffffff00 );
			healthDisplay.origin.x = healthDisplay.origin.y = 0;
			HUD.add( healthDisplay, true );
			
			// Show the title of the current wave and fade it out
			HUD.add( new WaveTitle( FlxG.width / 2 - 150, FlxG.height / 2 - 30, 300, "Wave " + wave.toString() ), true );
			
			// Create the game over text and set it to invisible
			gameOverText1 = new FlxText( FlxG.width / 2 - 200, FlxG.height / 2 - 30, 400, "GAME OVER" );
			gameOverText1.size = 50;
			gameOverText1.alignment = "center";
			gameOverText1.visible = false;
			HUD.add( gameOverText1, true );
			
			gameOverText2 = new FlxText( FlxG.width / 2 - 150, FlxG.height / 2 + 40, 300, "Press z or x to continue" );
			gameOverText2.size = 15;
			gameOverText2.alignment = "center";
			gameOverText2.visible = false;
			HUD.add( gameOverText2, true );
			
			super.create();
		}
		
		// Update the playstate
		override public function update():void
		{
			// Show HUD element
			scoreDisplay.text = FlxG.score.toString();
			healthDisplay.scale.x = playerShip.health;
			
			// If the player is dead, show the game over screen
			if ( playerShip.dead )
			{
				gameOverText1.visible = true;
				gameOverText2.visible = true;
				
				if ( fadeTimer > 0 )
				{
					fadeTimer -= FlxG.elapsed;
					if ( fadeTimer <= 0 )
					{
						wave = 1;
						FlxG.score = 0;
						FlxG.state = new StartState();
					}
				}
				else if ( FlxG.keys.Z || FlxG.keys.X )
				{
					FlxG.fade.start();
					fadeTimer = 1;
				}
			}			
			else if ( producers.countLiving() <= 0 )
			{
				// If all the producers are killed, start a new wave
				
				if ( fadeTimer > 0 )
				{
					fadeTimer -= FlxG.elapsed;
					if ( fadeTimer <= 0 )
					{
						wave++;
						FlxG.state = new PlayState();
					}
				}
				else
				{
					FlxG.fade.start();
					fadeTimer = 1;
				}
			}
			
			super.update();
		}
		
		// Spawn an enemy
		private function SpawnEnemy():Enemy
		{
			var validSpawnLocation:Boolean = false;
			var X:uint;
			var Y:uint;
			var deltaX:uint;
			var deltaY:uint;
			
			// Generate random locations until a good one is found
			while ( !validSpawnLocation )
			{
				// Get a random location within the play field
				X = Math.floor( Math.random() * PlayState.PLAYFIELD_SIZE );
				Y = Math.floor( Math.random() * PlayState.PLAYFIELD_SIZE );
				
				// Check that it is not spawning too close to the player
				deltaX = X - playerShip.x;
				deltaY = Y - playerShip.y;
				
				if ( deltaX * deltaX + deltaY * deltaY > SAFE_ZONE_SQUARED )
					validSpawnLocation = true;
			}
			
			return new Enemy( X, Y );
		}
		
	}

}