package  
{
	import org.flixel.*;
	
	/**
	 * ...
	 * @author Switchbreak
	 */
	public class PlayerShip extends FlxGroup
	{
		// Embed sounds and images
		[Embed( source = '../images/Pod.png' )] private var PodImg:Class;
		[Embed (source = "../sounds/hurt.mp3")] private var HurtSnd:Class;
		
		// Constants
		private const SPHERE_SIZE:uint					= 20;
		private const SPHERE_COLOR:uint					= 0xff00ff00;
		private const TURN_SPEED:Number					= 200;
		private const THRUST_ACCELERATION:Number		= 1000;
		private const THRUST_DRAG:Number				= 1000;
		private const MAX_THRUST:Number					= 40000;
		private const DISTANCE_SPEED:Number				= 80;
		private const DISTANCE_MAX:Number				= 100;
		private const STARTING_HEALTH:Number			= 100;
		private const ENEMY_DAMAGE:Number				= 10;
		private const ENEMY_RADIUS_SQUARED:Number		= 400;
		private const INVULNERABILITY_TIME:Number		= 2;
		private const PRODUCER_RADIUS_SQUARED:Number	= 841;
		private const LINE_RADIUS_SQUARED:Number		= 100;
		private const TRAIL_PARTICLES:uint				= 5;
		private const TRAIL_RANGE:uint					= 20;
		
		// Ship attributes
		public var leftPoint:FlxPoint;
		public var rightPoint:FlxPoint;
		private var leftSphere:FlxSprite;
		private var rightSphere:FlxSprite;
		private var line:FlxSprite;
		
		private var distance:Number;
		private var forward:Boolean = true;
		
		// Constructor
		public function PlayerShip(X:Number = 0, Y:Number = 0) 
		{
			// Call group constructor
			super();
			
			// Initialize attributes
			x = X;
			y = Y;
			distance = 0;
			angle = 90;
			health = STARTING_HEALTH;
			
			// Create connecting line
			line = new FlxSprite();
			line.createGraphic( 1, 1 );
			line.height = distance;
			add( line );
			
			// Create left and right sphere graphics
			leftPoint = new FlxPoint();
			rightPoint = new FlxPoint();
			
			leftSphere = new FlxSprite( 0, 0, PodImg )
			leftSphere.antialiasing = true;
			add( leftSphere );
			rightSphere = new FlxSprite( 0, 0, PodImg )
			rightSphere.antialiasing = true;
			add( rightSphere );
		}
		
		// Update player state
		override public function update():void
		{
			// No inertia affects on this ship, this is Star Wars physics
			velocity.x = velocity.y = 0;
			
			// Handle input
			if ( FlxG.keys.UP )
			{
				thrust += THRUST_ACCELERATION;
				if ( thrust > MAX_THRUST )
					thrust = MAX_THRUST;
				
				forward = true;
			}
			else if ( FlxG.keys.DOWN )
			{
				thrust -= THRUST_ACCELERATION;
				if ( thrust < -MAX_THRUST )
					thrust = -MAX_THRUST;
				
				forward = false;
			}
			else
			{
				// If no key is being pressed, decay the thrust
				if ( thrust > 0 )
				{
					thrust -= THRUST_DRAG;
					if ( thrust < 0 )
						thrust = 0;
				}
				else if( thrust < 0 )
				{
					thrust += THRUST_DRAG;
					if ( thrust > 0 )
						thrust = 0;
				}
			}
			
			if ( FlxG.keys.LEFT )
			{
				angularVelocity = -TURN_SPEED;
			}
			else if ( FlxG.keys.RIGHT )
			{
				angularVelocity = TURN_SPEED;
			}
			else
			{
				angularVelocity = 0;
			}
			
			if ( FlxG.keys.Z )
			{
				distance += DISTANCE_SPEED * FlxG.elapsed;
				if ( distance > DISTANCE_MAX )
					distance = DISTANCE_MAX;
			}
			if ( FlxG.keys.X )
			{
				distance -= DISTANCE_SPEED * FlxG.elapsed;
				if ( distance < 0 )
					distance = 0;
			}
			
			// Check that the ship is inside the playfield boundaries
			if ( x < 0 )
				x = 0;
			if ( y < 0 )
				y = 0;
			if ( x > PlayState.PLAYFIELD_SIZE - SPHERE_SIZE )
				x = PlayState.PLAYFIELD_SIZE - SPHERE_SIZE;
			if ( y > PlayState.PLAYFIELD_SIZE - SPHERE_SIZE )
				y = PlayState.PLAYFIELD_SIZE - SPHERE_SIZE;
			
			// Set sphere position and angle
			leftSphere.angle = rightSphere.angle = angle - 90;
			
			FlxU.rotatePoint( x, y - distance, x, y, angle, leftPoint )
			rightPoint.x = x - (leftPoint.x - x);
			rightPoint.y = y - (leftPoint.y - y);
			leftSphere.x = leftPoint.x - leftSphere.frameWidth / 2;
			leftSphere.y = leftPoint.y - leftSphere.frameHeight / 2;
			rightSphere.x = rightPoint.x - rightSphere.frameWidth / 2;
			rightSphere.y = rightPoint.y - rightSphere.frameHeight / 2;
			
			line.x = x;
			line.y = y;
			line.scale.y = distance * 2;
			line.angle = angle;
			
			// Check enemies in playfield for collisions
			for ( var i:uint = 0; i < PlayState.enemies.members.length; i++ )
			{
				var enemy:Enemy = PlayState.enemies.members[i];
				
				if ( enemy != null )
				{
					// Check for collisions with either of the two end-spheres
					var hit:Boolean = false;
					var centerX:Number = enemy.x + enemy.frameWidth / 2;
					var centerY:Number = enemy.y + enemy.frameHeight / 2;
					
					var deltaX:Number = centerX - leftPoint.x;
					var deltaY:Number = centerY - leftPoint.y;
					
					if ( deltaX * deltaX + deltaY * deltaY <= ENEMY_RADIUS_SQUARED )
						hit = true;
					
					deltaX = centerX - rightPoint.x;
					deltaY = centerY - rightPoint.y;
					
					if ( deltaX * deltaX + deltaY * deltaY <= ENEMY_RADIUS_SQUARED )
						hit = true;
					
					// If an end-sphere is hit and the player is not invulnerable, destroy the enemy
					// and take damage
					if( hit && !flickering() )
					{
						hurt( ENEMY_DAMAGE );
						enemy.Explode( false );
					}
					else
					{
						// If neither end-sphere was hit, check for collisions against the connecting
						// line
						
						// Get a vector from one of the line's endpoints to the enemy location
						var Ax:Number = centerX - leftPoint.x;
						var Ay:Number = centerY - leftPoint.y;
						
						// Get a vector from the same endpoint to the other line endpoint
						var Bx:Number = rightPoint.x - leftPoint.x;
						var By:Number = rightPoint.y - leftPoint.y;
						
						// Get the scalar projection of vector A onto B by taking the dot product and
						// dividing by magnitude of B
						var dot:Number = Ax * Bx + Ay * By;
						var u:Number = dot / (Bx * Bx + By * By);
						
						// Get coordinates of closest point on line segment to the enemy
						var lineX:Number;
						var lineY:Number;
						if ( u < 0 )
						{
							// If the projection is <0, the closest point on the line segment is
							// endpoint 1
							lineX = leftPoint.x;
							lineY = leftPoint.y;
						}
						else if ( u > 1 )
						{
							// If the projection is >1, the closest point on the line segment is
							// endpoint 2
							lineX = rightPoint.x;
							lineY = rightPoint.y;
						}
						else
						{
							// Get the point inside the line segment indicated by u
							lineX = leftPoint.x + u * Bx;
							lineY = leftPoint.y + u * By;
						}
						
						// Get the distance between the closest point on the line and the enemy
						deltaX = lineX - centerX;
						deltaY = lineY - centerY;							
						if ( deltaX * deltaX + deltaY * deltaY <= LINE_RADIUS_SQUARED )
						{
							// If an enemy hit the line, throw them backwards
							if( forward )
								enemy.Throw( angle + 90 );
							else
								enemy.Throw( angle + 270 );
						}
					}
				}
			}
			
			// Check producers in playfield for collisions
			for ( i = 0; i < PlayState.producers.members.length; i++ )
			{
				var producer:Producer = PlayState.producers.members[i];
				
				if ( producer != null )
				{
					// Check for collisions with either of the two end-spheres
					hit = false;
					centerX = producer.x + producer.frameWidth / 2;
					centerY = producer.y + producer.frameHeight / 2;
					
					deltaX = centerX - leftPoint.x;
					deltaY = centerY - leftPoint.y;
					
					if ( deltaX * deltaX + deltaY * deltaY <= PRODUCER_RADIUS_SQUARED )
						hit = true;
					
					deltaX = centerX - rightPoint.x;
					deltaY = centerY - rightPoint.y;
					
					if ( deltaX * deltaX + deltaY * deltaY <= PRODUCER_RADIUS_SQUARED )
						hit = true;
					
					// If an end-sphere is hit and the player is not invulnerable, destroy the enemy
					// and take damage
					if( hit && !flickering() )
					{
						hurt( ENEMY_DAMAGE );
					}
				}
			}
			
			super.update();
		}
		
		override public function hurt( Damage:Number ):void
		{
			FlxG.flash.start();
			FlxG.play( HurtSnd );
			flicker( INVULNERABILITY_TIME );
			
			super.hurt( Damage );
		}
		
	}

}