package  
{
	import org.flixel.*;
	
	/**
	 * ...
	 * @author Switchbreak
	 */
	public class Producer extends FlxSprite
	{
		// Embed sounds and images
		[Embed( source = '../images/Producer.png' )] private var ProducerImg:Class;
		[Embed (source = "../sounds/producer-hit.mp3")] private var ExplosionSnd:Class;
		[Embed (source = "../sounds/producer-dead.mp3")] private var DeathSnd:Class;
		
		// Constants
		private const SPAWN_RATE:Number			= 5;
		private const LAST_SPAWN_RATE:Number	= 1;
		private const PRODUCER_SIZE:uint		= 40;
		private const PRODUCER_COLOR:uint		= 0xffffffff;
		private const STARTING_HEALTH:uint		= 20;
		private const HEALTH_BAR_COLOR:uint		= 0xffffff00;
		private const POINTS:uint				= 1000;
		private const EXPLODE_PARTICLES:uint	= 10;
		
		// Producer attributes
		private var spawnTimer:Number = SPAWN_RATE;
		private var healthBar:FlxSprite;
		
		public function Producer(X:Number = 0, Y:Number = 0) 
		{
			super(X, Y, ProducerImg);
			
			x -= width / 2;
			y -= height / 2;
			
			health = STARTING_HEALTH;
			
			healthBar = new FlxSprite( X - STARTING_HEALTH, y - 15 );
			healthBar.createGraphic( 1, 5, HEALTH_BAR_COLOR );
			healthBar.origin.x = healthBar.origin.y = 0;
			healthBar.scale.x = health * 2;
			FlxG.state.add( healthBar );
		}
		
		override public function update():void
		{
			// Countdown to next enemy spawn
			spawnTimer -= FlxG.elapsed;
			if ( spawnTimer <= 0 )
			{
				PlayState.enemies.add( new Enemy( x, y ) );
				if( PlayState.producers.countLiving() > 1 )
					spawnTimer = SPAWN_RATE;
				else
					spawnTimer = LAST_SPAWN_RATE;
			}
		}
		
		override public function hurt( Damage:Number ):void
		{
			super.hurt( Damage );
			FlxG.quake.start( 0.01 );
			
			// Create explosion particle effect
			var explode:FlxEmitter = new FlxEmitter( x + frameWidth / 2, y + frameHeight / 2 );
			explode.gravity = 0;
			for ( var i:uint = 0; i < EXPLODE_PARTICLES; i++ )
			{
				var newParticle:FlxSprite = new FlxSprite( x, y );
				newParticle.createGraphic( 5, 5, PRODUCER_COLOR );
				explode.add( newParticle );
			}
			FlxG.state.add( explode );
			explode.start();
			
			if ( !dead )
			{
				FlxG.play( ExplosionSnd );
				healthBar.scale.x = health * 2;
			}
		}
		
		override public function kill():void
		{
			healthBar.kill();
			super.kill();
			
			FlxG.play( DeathSnd );
			FlxG.score += POINTS;
			FlxG.state.add( new ScoreText( x, y, 100, "+" + POINTS.toString() ) );
			PlayState.producers.remove( this );
		}
		
	}

}